/* Copyright John-Philip "Seriema" Johansson (seriema@users.sourceforge.net)
 * released under the MIT license */


using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace ThumbviewConnector
{
    internal delegate bool GetInfoDelegate(string path, out PixelFormat format, out System.Drawing.Size size);
    internal delegate IntPtr ExtractThumbnailDelegate(string path, PixelFormat recomendedBPP, ref System.Drawing.Size requiredSize);
    internal delegate IntPtr SupportedFiletypesDelegate();
    internal delegate string VersionDelegate();
    internal delegate string NameDelegate();

    public enum PixelFormat
    {
        Palette = 8,
        RGB24 = 24,
        RGBA32 = 32
    }

    public sealed class ExtractorConnector : IDisposable
    {

        #region Constructor, destructor and disposer

        public ExtractorConnector(string libraryName)
        {
            library = new Kent.Library.DynamicLibrary(libraryName);

            library.GetDelegateByName("GetInfo", out getInfoMethod);
            library.GetDelegateByName("ExtractThumbnail", out extractThumbnailMethod);
            library.GetDelegateByName("SupportedFiletypes", out supportedFileTypesMethod);
            library.GetDelegateByName("Version", out versionMethod);
            library.GetDelegateByName("Name", out nameMethod);
        }

        ~ExtractorConnector()
        {
            Dispose(false);
        }

        private void Dispose(bool disposing)
        {
            if (!disposed && disposing)
            {
                library.Dispose();
                disposed = true;
            }
        }

        void IDisposable.Dispose()
        {
            Dispose(true);
        }

        #endregion

        #region Extractor information !

        /// <summary>
        /// The version of the called ThumbView Extractor.
        /// </summary>
        /// <returns>The version number as a null terminated string. It is correctly string-comparable to other versions.</returns>
        public Version Version { get { return new Version(versionMethod()); } }

        /// <summary>
        /// The name of the called ThumbView Extractor.
        /// </summary>
        /// <returns>The name of this Extractor as a null terminated string.</returns>
        public string Name { get { return nameMethod(); } }

        /// <summary>
        /// The filetypes are supported by the ThumbView Extractor.
        /// </summary>
        /// <returns>A array of strings to supported file extensions, each file extension is a lowercase string and excludes the dot ('.') character.</returns>
        public string[] SupportedFileTypes
        {
            get
            {
                if (supportedFileTypes == null)
                {
                    GetSupportedFileTypes();
                }
                return supportedFileTypes;
            }
        }

        #endregion

        #region Image processing and information !

        /// <summary>
        /// Creates a thumbnail from the given image. Note: Does not stretch the image.
        /// </summary>
        /// <param name="Path">Zero terminated file path to the image.</param>
        /// <param name="RecomendedBPP">Recommended color depth in units of bits per pixel.</param>
        /// <param name="RequiredSize">Desired width and height of the image.</param>
        /// <returns>A handle to a System.Drawing.Bitmap on success, NULL on failure.</returns>
        public System.Drawing.Bitmap ExtractThumbnail(string path, PixelFormat recommendedFormat, ref System.Drawing.Size requiredSize)
        {
            IntPtr thumbnail = extractThumbnailMethod(path, recommendedFormat, ref requiredSize);
            if (thumbnail == IntPtr.Zero)
            {
                return null;
            }

            return System.Drawing.Bitmap.FromHbitmap(thumbnail);
        }

        /// <summary>
        /// Collects basic information about the givem image.
        /// </summary>
        /// <param name="Path">File path to the image as a string.</param>
        /// <param name="BPP">Original color depth in units of bits per pixel.</param>
        /// <param name="Size">Original width and height of the image.</param>
        /// <returns>False on error, otherwise true.</returns>
        public bool GetInfo(string path, out PixelFormat format, out System.Drawing.Size size)
        {
            return getInfoMethod(path, out format, out size);
        }

        #endregion

        #region Support methods

        private void GetSupportedFileTypes()
        {
            List<string> types = new List<string>();
            IntPtr stringArray = supportedFileTypesMethod();

            unsafe
            {
                IntPtr stringPtr = DereferenceArrayPointer(stringArray);

                while (stringPtr != IntPtr.Zero)
                {
                    string s = Marshal.PtrToStringAnsi(stringPtr);
                    types.Add(s);

                    stringArray = new IntPtr(stringArray.ToInt64() + Marshal.SizeOf(typeof(IntPtr)));
                    stringPtr = DereferenceArrayPointer(stringArray);
                }
            }

            supportedFileTypes = types.ToArray();
        }

        unsafe private static IntPtr DereferenceArrayPointer(IntPtr stringArray)
        {
            return new IntPtr(*(int**)stringArray.ToPointer());
        }

        #endregion


        private GetInfoDelegate getInfoMethod;
        private ExtractThumbnailDelegate extractThumbnailMethod;
        private SupportedFiletypesDelegate supportedFileTypesMethod;
        private VersionDelegate versionMethod;
        private NameDelegate nameMethod;
        private string[] supportedFileTypes;
        private bool disposed = false;
        private Kent.Library.DynamicLibrary library;
    }
}
